@@ -6,7 +6,7 @@
  * Phenoelit (http://www.phenoelit.de)
  * (c) 2k
  *
- * $Id: ass_v1.c,v 1.20 2001/07/08 13:14:16 fx Exp $
+ * $Id: ass_v1.c,v 1.23 2001/10/21 17:12:08 fx Exp fx $
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +39,7 @@
 #define CAPLENGTH	1542
 #define DEST_LENGTH	15
 
-#define BANNER		"ASS [Autonomous System Scanner] $Revision: 1.20 $\n"\
+#define BANNER		"ASS [Autonomous System Scanner] $Revision: 1.23 $\n"\
 			"\t(c) 2k++ FX <fx@phenoelit.de>\n"\
 			"\tPhenoelit (http://www.phenoelit.de)\n"
 
@@ -151,6 +151,20 @@ typedef struct {
     void		*next;
 } RES_route_ospf_t;
 
+typedef struct {
+    char		*deviceid;
+//    struct in_addr	addr;
+    char		*portid;
+    unsigned long	capas;
+    char		*software;
+    char		*platform;
+    char		*vtpmgmt;
+    unsigned int	vlan;
+    unsigned short	duplex;
+    void		*next;
+} RES_cdp_info_t;
+
+
 #define CAP_IGRP	0x001
 #define CAP_IRDP	0x002
 #define CAP_EIGRP	0x004
@@ -175,6 +189,7 @@ typedef struct {
     RES_route_eigrprt_t	*eigrpr;
     RES_route_hsrp_t	*hsrp;
     RES_route_ospf_t	*ospf;
+    RES_cdp_info_t	*cdp;
     void		*next;		// next record
 } result_t;
 
@@ -197,6 +212,7 @@ struct {
     int			passive;
     unsigned long	sdelay;
     char		*active_protos;
+    char		*inputfile;
 } cfg;
 
 /************************************
@@ -253,7 +269,7 @@ int	main(int argc, char **argv) {
 
     memset(&cfg,0,sizeof(cfg));
     cfg.passive=cfg.prom=1;
-    while ((option=getopt(argc,argv,"vcpAMi:a:b:S:D:T:P:"))!=EOF) {
+    while ((option=getopt(argc,argv,"vcpAMi:a:b:S:D:T:P:r:"))!=EOF) {
 	switch (option) {
 	    case 'v':	/* verbose */
 			cfg.verbose++;
@@ -303,11 +319,15 @@ int	main(int argc, char **argv) {
 			cfg.active_protos=smalloc(strlen(optarg)+1);
 			strcpy(cfg.active_protos,optarg);
 			break;
+	    case 'r':	/* input file */
+			cfg.inputfile=(char*)smalloc(strlen(optarg)+1);
+			strcpy(cfg.inputfile,optarg);
+			break;
 	    default:	usage(argv[0]);
 	}
     }
 
-    if (!cfg.device) usage(argv[0]);
+    if ((!cfg.device)&&(!cfg.inputfile)) usage(argv[0]);
     if (!cfg.set_dest)
 	inet_aton(IP_BCAST,&(cfg.dest));
 
@@ -323,19 +343,26 @@ int	main(int argc, char **argv) {
 	strcpy(cfg.active_protos,FCHAR_ACTIVE_ALL);
     }
 
+    if (cfg.inputfile!=NULL) {
+	if (!cfg.passive) 
+	    printf("Passive mode implicit when reading from file\n");
+	cfg.passive=1; cfg.cont=0;
+    }
 
-    /* set up socket ... */
-    if ((atsock=init_socket_IP4(cfg.device,1))==(-1)) return(1);
-    /* if spoofing is enabled, copy it */
-    if (!cfg.spoof_src) {
-	memcpy(&(cfg.src.s_addr), &(packet_ifconfig.ip.s_addr), IP_ADDR_LEN);
+    if (!cfg.inputfile) {
+	/* set up socket ... */
+	if ((atsock=init_socket_IP4(cfg.device,1))==(-1)) return(1);
+	/* if spoofing is enabled, copy it */
+	if (!cfg.spoof_src) {
+	    memcpy(&(cfg.src.s_addr), &(packet_ifconfig.ip.s_addr), IP_ADDR_LEN);
+	}
     }
     /* set up sniffer */
     if (initialize_pcap()==(-1)) return (1);
 
     /* signal handling */
     signal(SIGTERM,&signaler);
-    signal(SIGABRT,&signaler);
+    //signal(SIGABRT,&signaler);
     signal(SIGINT,&signaler);
 
     /* my shit */
@@ -488,11 +515,14 @@ int	main(int argc, char **argv) {
 	printf("\n");
     }
 
+    print_results();
+
     /* at the end of the day, close our socket */
     pcap_close(cap);
-    close(atsock);
+    if (!cfg.inputfile) {
+	close(atsock);
+    }
 
-    print_results();
     clean_results();
 
     return (0);
@@ -510,6 +540,7 @@ void 	net_listen(void) {
     if ((pcap_data=(u_char *)pcap_next(cap,&pcap_head))!=NULL) {
 	/* make a local copy of the data, 
 	 * pcap will overwrite the buffer if needed */
+	
 	memcpy(&phead,&pcap_head,sizeof(struct pcap_pkthdr));
 	ppacket=(u_char *)smalloc(phead.caplen);
 	memcpy(ppacket,pcap_data,phead.caplen);
@@ -517,7 +548,7 @@ void 	net_listen(void) {
 	evaluate_packet(ppacket,phead.caplen);
 
 	free(ppacket);
-    }
+    } else if (cfg.inputfile) stop_flag++;
 }
 
 void	evaluate_packet(u_char *frame,int frame_length) {
@@ -540,8 +571,11 @@ void	evaluate_packet(u_char *frame,int frame_length) {
 
     /* for CDP */
     struct eth_LLC		*eth_llc;
-    u_char			*cdpt;
+    u_char			*cdpbegin;
+    cdp_generic_t		*cdpg;
     struct cdp_address_entry	*cdpa;
+    unsigned int		cdpdatalength;
+    struct in_addr 		caa;
     
     int				i;
     
@@ -836,18 +870,25 @@ void	evaluate_packet(u_char *frame,int frame_length) {
 	if (cfg.verbose>1) { printf("%c",FCHAR_CDP); fflush(stdout); } 
 	else if (cfg.verbose>2) printf("CDP packet ...\n");
 
-	cdpt=(u_char *)(frame
+	cdpbegin=(u_char *)(frame
 		+sizeof(struct eth_ieee802_3)
 		+sizeof(struct eth_LLC)
 		+sizeof(struct cdphdr));
+	cdpdatalength=(frame_length
+		-sizeof(struct eth_ieee802_3)
+		-sizeof(struct eth_LLC)
+		-sizeof(struct cdphdr));
+	cdpg=(cdp_generic_t*)cdpbegin;
 
 	do {
-	    if (ntohs(*((u_int16_t *)cdpt))==TYPE_ADDRESS) {
+	    if (ntohs(cdpg->type)==TYPE_ADDRESS) {
 		if (cfg.verbose>2) 
 		    printf("Number of addr's in it: %d\n",
-			ntohl(*((u_int32_t *)(cdpt+2*sizeof(u_int16_t)))));
+			ntohl(*((u_int32_t *)
+				((u_char*)cdpg+2*sizeof(u_int16_t)))
+			    ));
 
-		cdpa=(struct cdp_address_entry *)(cdpt+
+		cdpa=(struct cdp_address_entry *)((u_char*)cdpg+
 			2*sizeof(u_int16_t)+sizeof(u_int32_t));
 		if (cdpa->proto!=0xCC) {
 		    if (cfg.verbose) printf("CDP not speaking IP at"
@@ -858,23 +899,17 @@ void	evaluate_packet(u_char *frame,int frame_length) {
 			    *((u_char*)frame+9)&0xFF,
 			    *((u_char*)frame+10)&0xFF,
 			    *((u_char*)frame+11)&0xFF);
+		    memset(&(caa),0,sizeof(caa));
+		    add_route(CAP_CDP,&caa,cdpdatalength,(u_char*)cdpbegin); 
 		} else {
-		    struct in_addr caa;
-
 		    memcpy(&(caa.s_addr),&(cdpa->addr),IP_ADDR_LEN);
-		    add_route(CAP_CDP,&caa,0,NULL); 
+		    add_route(CAP_CDP,&caa,cdpdatalength,(u_char*)cdpbegin); 
 		}
 	    }
-	    cdpt+=htons(
-		    *((u_int16_t *)((u_char*)cdpt+sizeof(u_int16_t)))
-		    );
+	    
+	    cdpg=(cdp_generic_t *)((u_char*)cdpg+ntohs(cdpg->length));
 
-	} while (
-		((u_char*)cdpt-(u_char*)frame)
-		+htons(
-		    *((u_int16_t *)((u_char*)cdpt+sizeof(u_int16_t)))
-		    )
-		<frame_length);
+	} while (((u_char*)cdpg-(u_char*)frame)<frame_length); 
 	/* means: loop as long through the CDP sections until the pointer
 	 * would reach outside the packet */
     }
@@ -887,7 +922,7 @@ void	signaler(int sig) {
     stop_flag++;
     if (cfg.verbose>2)
 	fprintf(stderr,"\nSignal received.\n");
-    pcap_close(cap);
+    //pcap_close(cap);
 }
 
 
@@ -901,44 +936,52 @@ int	initialize_pcap(void) {
     char		tipstr[IPSTRLEN+1];
     char		*notfilter;
 
-    /* prepare filter */
-    memset(&tipstr,0,IPSTRLEN+1);
-    strcpy(tipstr,inet_ntoa(cfg.src));
-    notfilter=(char *)smalloc(strlen(PATTERNSTRING)+IPSTRLEN+1);
-    strcpy(notfilter,PATTERNSTRING);
-    strcat(notfilter,tipstr);
+    if (!cfg.inputfile) {
+	/* prepare filter */
+	memset(&tipstr,0,IPSTRLEN+1);
+	strcpy(tipstr,inet_ntoa(cfg.src));
+	notfilter=(char *)smalloc(strlen(PATTERNSTRING)+IPSTRLEN+1);
+	strcpy(notfilter,PATTERNSTRING);
+	strcat(notfilter,tipstr);
 
-    /* get my network and netmask */
-    if (pcap_lookupnet(cfg.device,&network,&netmask,pcap_err)!=0) {
-	fprintf(stderr,"pcap_lookupnet(): %s\n",pcap_err);
-	return (-1);
+	/* get my network and netmask */
+	if (pcap_lookupnet(cfg.device,&network,&netmask,pcap_err)!=0) {
+	    fprintf(stderr,"pcap_lookupnet(): %s\n",pcap_err);
+	    return (-1);
+	}
+
+	/* open the sniffer */
+	if ((cap=pcap_open_live(cfg.device,CAPLENGTH,
+			cfg.prom, /* in promi mode */
+			0, /* no timeouts sometimes don't work ...*/
+			pcap_err))==NULL) {
+	    fprintf(stderr,"pcap_open_live(): %s\n",pcap_err);
+	    return (-1);
+	}
+
+	if (pcap_datalink(cap)!=DLT_EN10MB) {
+	    fprintf(stderr,"works on Ethernet only, sorry.\n");
+	    return (-1);
+	}
+
+	if (pcap_compile(cap,&cfilter,notfilter,0,netmask)!=0) {
+	    pcap_perror(cap,"pcap_compile()");
+	    return (-1);
+	}
+
+	if (pcap_setfilter(cap,&cfilter)!=0) {
+	    pcap_perror(cap,"pcap_setfilter()");
+	    return (-1);
+	}
+	
+	free(notfilter);
+    } else {
+	if ((cap=pcap_open_offline(cfg.inputfile,pcap_err))==NULL) {
+	    fprintf(stderr,"pcap_open_offline(): %s\n",pcap_err);
+	    return (-1);
+	}
     }
 
-    /* open the sniffer */
-    if ((cap=pcap_open_live(cfg.device,CAPLENGTH,
-		    cfg.prom, /* in promi mode */
-		    0, /* no timeouts sometimes don't work ...*/
-		    pcap_err))==NULL) {
-	fprintf(stderr,"pcap_open_live(): %s\n",pcap_err);
-	return (-1);
-    }
-
-    if (pcap_datalink(cap)!=DLT_EN10MB) {
-	fprintf(stderr,"works on Ethernet only, sorry.\n");
-	return (-1);
-    }
-
-    if (pcap_compile(cap,&cfilter,notfilter,0,netmask)!=0) {
-	pcap_perror(cap,"pcap_compile()");
-	return (-1);
-    }
-
-    if (pcap_setfilter(cap,&cfilter)!=0) {
-	pcap_perror(cap,"pcap_setfilter()");
-	return (-1);
-    }
-    
-    free(notfilter);
     return 0;
 }
 
@@ -1215,13 +1258,28 @@ void	usage(char *n) {
 	    "%s [-v[v[v]]] -i <interface> [-A] [-p] [-c] [-M] [-P IER12]\n"
 	    "\t[-a <autonomous system start> -b <autonomous system stop>]\n"
 	    "\t[-S <spoofed source IP>] [-D <destination ip>]\n"
-	    "\t[-T <packets per delay>]\n",
+	    "\t[-T <packets per delay>]\n"
+	    "\t[-r <filename>]\n",
 	    n);
     exit (1);
 }
 
 
 int	add_route(int rtype,struct in_addr *addr,unsigned int as,void *data) {
+    /* For all routing protocols, the handling is as follows:
+     * 		- look for an entry in the result_t list that has the same 
+     * 		  address. If there is such an entry, use this. Otherwise 
+     * 		  create one. 
+     * 		- For the particular protocol, check if the information is 
+     * 		  already in there (based on fields depending on the protocol
+     * 		  such as the AS) and add the info in the tree as needed.
+     * 		- CDP is different. It works the same way for devices running 
+     * 		  CDP and IP - but devices not running IP (such as switches)
+     * 		  will be included as router 0.0.0.0. The CDP subtree looks for 
+     * 		  the device name as key (Oops - this is what cisco did ;)
+     * 		  WARNING: variable "as" is used to transport the frame length
+     * 		  minus Ethernet,LLC and CdpHdr !!!
+     */
     result_t		*current,*c2;
     RES_route_igrp_t	*c_igrp;
     RES_route_irdp_t	*c_irdp;
@@ -1231,6 +1289,7 @@ int	add_route(int rtype,struct in_addr *addr,unsigned int as,void *data) {
     RES_route_eigrprt_t	*c_eigrpr;
     RES_route_hsrp_t	*c_hsrp;
     RES_route_ospf_t	*c_ospf;
+    RES_cdp_info_t	*c_cdp,*c2_cdp;
     igrp_system_entry_t	*isys;
     irdp_rec_t		*irdpr;
     ripv1addr_t		*rip1a;
@@ -1242,11 +1301,13 @@ int	add_route(int rtype,struct in_addr *addr,unsigned int as,void *data) {
     hsrp_t		*hsrp;
     ospf_header_t	*ospfh;
     ospf_hello_t	*ospfl;
+    cdp_generic_t	*cdprec;
     u_int32_t		temp;
     int			i;
 
     current=anchor;
     c2=NULL;
+
     /* look for already exisiting one */
     while (current!=NULL) {
 	if (!memcmp(&(current->addr),addr,sizeof(struct in_addr))) break;
@@ -1573,7 +1634,7 @@ int	add_route(int rtype,struct in_addr *addr,unsigned int as,void *data) {
 	     */
 	case CAP_OSPF:
 	    current->capa=current->capa|CAP_OSPF;
-	    ospfh=(ospf_header_t *)data;
+	    if ((ospfh=(ospf_header_t *)data)==NULL) break;
 
 	    if (current->ospf==NULL) {
 		current->ospf=(RES_route_ospf_t *)
@@ -1663,7 +1724,90 @@ int	add_route(int rtype,struct in_addr *addr,unsigned int as,void *data) {
 	     * CDP
 	     */
 	case CAP_CDP:
+	    c_cdp=(RES_cdp_info_t *)smalloc(sizeof(RES_cdp_info_t));
+	    /* data should point to the first TLV record in the frame */
+	    cdprec=(cdp_generic_t *)data;
+
+	    /* at CDP, we dissect the packet first and add it later */
+	    do {
+		if (cfg.verbose>2) 
+		    printf("CDP add: type %u, length %u\n",
+			    ntohs(cdprec->type),ntohs(cdprec->length));
+
+		if (ntohs(cdprec->type)==TYPE_DEVICE_ID) {
+		    c_cdp->deviceid=(char*)smalloc(ntohs(cdprec->length)+1);
+		    memcpy(c_cdp->deviceid,&(cdprec->value),
+			    ntohs(cdprec->length));
+		} else if (ntohs(cdprec->type)==TYPE_PORT_ID) {
+		    c_cdp->portid=(char*)smalloc(ntohs(cdprec->length)+1);
+		    memcpy(c_cdp->portid,&(cdprec->value),
+			    ntohs(cdprec->length));
+		} else if (ntohs(cdprec->type)==TYPE_CAPABILITIES) {
+		    c_cdp->capas=(unsigned long)
+			ntohl(*((u_int32_t *)&(cdprec->value)));
+		} else if (ntohs(cdprec->type)==TYPE_IOS_VERSION) {
+		    c_cdp->software=(char*)smalloc(ntohs(cdprec->length)+1);
+		    memcpy(c_cdp->software,&(cdprec->value),
+			    ntohs(cdprec->length));
+		} else if (ntohs(cdprec->type)==TYPE_PLATFORM) {
+		    c_cdp->platform=(char*)smalloc(ntohs(cdprec->length)+1);
+		    memcpy(c_cdp->platform,&(cdprec->value),
+			    ntohs(cdprec->length));
+		} else if (ntohs(cdprec->type)==TYPE_VTP_MGMT) {
+		    c_cdp->vtpmgmt=(char*)smalloc(ntohs(cdprec->length)+1);
+		    memcpy(c_cdp->vtpmgmt,&(cdprec->value),
+			    ntohs(cdprec->length));
+		} else if (ntohs(cdprec->type)==TYPE_VLAN) {
+		    c_cdp->vlan=ntohs(*((u_int16_t*)&(cdprec->value)));
+		} else if (ntohs(cdprec->type)==TYPE_DUPLEX) {
+		    c_cdp->duplex=(u_int8_t)cdprec->value;
+		} else {
+		    if (cfg.verbose>2) {
+			fprintf(stderr,"Unknown CDP type %u\n",
+				ntohs(cdprec->type));
+			fprintf(stderr,"DUMP: %02X %02X %02X %02X %02X %02X\n",
+				*((u_int8_t *)cdprec),
+				*((u_int8_t *)cdprec+1),
+				*((u_int8_t *)cdprec+2),
+				*((u_int8_t *)cdprec+3),
+				*((u_int8_t *)cdprec+4),
+				*((u_int8_t *)cdprec+5));
+		    }
+		}
+		cdprec=(cdp_generic_t *)((u_char*)cdprec+ntohs(cdprec->length));
+		/* printf("DEBUG: data = %p, cdprec = %p, cdprec-data = %u,"
+			" length = %u\n",
+			data,(u_char *)cdprec,((u_char*)cdprec-(u_char*)data),as);
+			*/
+	    } while ( ((u_char*)cdprec-(u_char*)data)<as);
+
 	    current->capa=current->capa|CAP_CDP;
+	    if (current->cdp==NULL) {
+		current->cdp=c_cdp;
+	    } else {
+		c2_cdp=current->cdp;
+		while (c2_cdp!=NULL) {
+		    if (c_cdp->deviceid!=NULL) {
+			if (!strcmp(c2_cdp->deviceid,c_cdp->deviceid)){
+			    // source already known for this device 
+			    if (cfg.verbose>2) printf("CDP: Device %s"
+				    " alread known\n",c_cdp->deviceid);
+			    if (c_cdp->deviceid!=NULL) free(c_cdp->deviceid);
+			    if (c_cdp->portid!=NULL) free(c_cdp->portid);
+			    if (c_cdp->software!=NULL) free(c_cdp->software);
+			    if (c_cdp->platform!=NULL) free(c_cdp->platform);
+			    if (c_cdp->vtpmgmt!=NULL) free(c_cdp->vtpmgmt);
+			    free(c_cdp);
+			    return 1;
+			}
+		    }
+		    c2_cdp=c2_cdp->next;
+		}
+		// go to the last valid entry 
+		c2_cdp=current->cdp;
+		while (c2_cdp->next!=NULL) c2_cdp=c2_cdp->next;
+		c2_cdp->next=c_cdp;
+	    } 
 	    break;
 
 	default:
@@ -1686,6 +1830,7 @@ void	print_results(void) {
     RES_route_eigrprt_t	*c_eigrpr;
     RES_route_hsrp_t	*c_hsrp;
     RES_route_ospf_t	*c_ospf;
+    RES_cdp_info_t	*c_cdp;
 
     printf("\n\n>>>Results>>>\n");
 
@@ -1704,6 +1849,47 @@ void	print_results(void) {
 	if (current->capa&CAP_OSPF) printf("OSPF ");
 	printf(")\n");
 
+	/* CDP results */
+	c_cdp=current->cdp;
+	while(c_cdp!=NULL) {
+	    printf("\tCDP [ n/a ]  %17s  %s\n","Device ID",c_cdp->deviceid);
+	    if (c_cdp->portid!=NULL);
+		printf("\t%13s%17s  %s\n"," ","Port ID",c_cdp->portid);
+	    if (c_cdp->platform!=NULL)
+		printf("\t%13s%17s  %s\n"," ","Platform",c_cdp->platform);
+	    if (c_cdp->vtpmgmt!=NULL)
+		printf("\t%13s%17s  %s\n"," ","VTP Mgmt",c_cdp->vtpmgmt);
+
+	    if ((c_cdp->capas&CDP_CAP_LEVEL3_ROUTER)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Layer 3 Router");
+	    if ((c_cdp->capas&CDP_CAP_LEVEL2_TRBR)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Layer 2 Transparent Bridge");
+	    if ((c_cdp->capas&CDP_CAP_LEVEL2_SRB)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Layer 2 SrcRoute Bridge");
+	    if ((c_cdp->capas&CDP_CAP_LEVEL2_SWITCH)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Layer 2 Switch");
+	    if ((c_cdp->capas&CDP_CAP_NETWORK_LAYER)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Network Layer Protocols");
+	    if ((c_cdp->capas&CDP_CAP_FORWARD_IGMP)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Fwds IGMP to non-routers");
+	    if ((c_cdp->capas&CDP_CAP_LEVEL1)!=0)
+		printf("\t%13s%17s  %s\n"," "," ","- Provides Level 1 funct");
+
+	    if (c_cdp->vlan!=0) 
+		printf("\t%13s%17s  %u\n"," ","Native VLAN",c_cdp->vlan);
+
+	    if (c_cdp->duplex==1) 
+		printf("\t%13s%17s  %s\n"," ","Duplex","FULL");
+	    else if (c_cdp->duplex==0) 
+		printf("\t%13s%17s  %s\n"," ","Duplex","Half");
+	    else 
+		printf("\t%13s%17s  %u\n"," ","Duplex",c_cdp->duplex);
+
+	    if ((cfg.verbose)&&(c_cdp->software!=NULL))
+		printf("\t%13s%17s \n%s\n"," ","Software ID",c_cdp->software);
+
+	    c_cdp=c_cdp->next;
+	}
 	/* display all IGRP results */
 	c_igrp=current->igrp;
 	while (c_igrp!=NULL) {
@@ -1877,6 +2063,7 @@ void	clean_results(void) {
     RES_route_eigrprt_t	*k_eigrpr,*c_eigrpr;
     RES_route_hsrp_t	*k_hsrp,*c_hsrp;
     RES_route_ospf_t	*k_ospf,*c_ospf;
+    RES_cdp_info_t	*k_cdp,*c_cdp;
 
     current=anchor;
     while (current!=NULL) {
@@ -1929,6 +2116,17 @@ void	clean_results(void) {
 	    c_ospf=c_ospf->next;
 	    free(k_ospf);
 	}
+	c_cdp=current->cdp;
+	while (c_cdp!=NULL) {
+	    k_cdp=c_cdp;
+	    c_cdp=c_cdp->next;
+	    if (k_cdp->deviceid!=NULL) free(k_cdp->deviceid);
+	    if (k_cdp->portid!=NULL) free(k_cdp->portid);
+	    if (k_cdp->software!=NULL) free(k_cdp->software);
+	    if (k_cdp->platform!=NULL) free(k_cdp->platform);
+	    if (k_cdp->vtpmgmt!=NULL) free(k_cdp->vtpmgmt);
+	    free(k_cdp);
+	}
 
 	k=current;
 	current=current->next;
