@@ -1,7 +1,7 @@
 /* Phenoelit IRPAS 
  * protocol definitions
  *
- * $Id: protocols.h,v 1.8 2001/07/07 12:48:53 fx Exp $
+ * $Id: protocols.h,v 1.9 2001/10/18 12:02:58 fx Exp $
  */
 #ifndef _PROTOCOLS_H_
 #define _PROTOCOLS_H_
@@ -59,6 +59,9 @@ struct cdphdr {
 #define TYPE_CAPABILITIES       0x0004
 #define TYPE_IOS_VERSION        0x0005
 #define TYPE_PLATFORM           0x0006
+#define TYPE_VTP_MGMT		0x0009
+#define TYPE_VLAN		0x000a
+#define TYPE_DUPLEX		0x000b
 
 struct cdp_device {
     u_int16_t           type;           /* 0x0001 */
@@ -86,6 +89,7 @@ struct cdp_port {
     u_char              port;           /* pointer to port name */
 };
 
+
 #define CDP_CAP_LEVEL1          0x40
 #define CDP_CAP_FORWARD_IGMP    0x20
 #define CDP_CAP_NETWORK_LAYER   0x10
@@ -111,6 +115,12 @@ struct cdp_platform {
     u_char              platform;       /* pointer to platform string */
 };
 
+typedef struct {
+    u_int16_t           type;           
+    u_int16_t           length;
+    u_char              value;           /* pointer to port name */
+} cdp_generic_t;
+
 /* ************************************************************
  * ARP version 4
  * ************************************************************/
@@ -294,6 +304,14 @@ typedef struct {
     u_int8_t	mask[4]			__attribute__ ((packed));
 } icmp_netmask_t;
 
+typedef struct {
+    u_int16_t   identifier		__attribute__ ((packed));
+    u_int16_t   seq			__attribute__ ((packed));
+    u_int32_t   origts			__attribute__ ((packed));
+    u_int32_t   recvts			__attribute__ ((packed));
+    u_int32_t   transts			__attribute__ ((packed));
+} icmp_timestamp_t;
+
 typedef struct {
     u_int8_t	num_addr		__attribute__ ((packed));
     u_int8_t	addrsize		__attribute__ ((packed));
@@ -352,6 +370,7 @@ typedef struct {
     u_int8_t	dead_interval[4] 	__attribute__ ((packed));
     u_int8_t	designated[4]		__attribute__ ((packed));
     u_int8_t	backup[4]		__attribute__ ((packed));
+    //u_int8_t	activeneig[4]		__attribute__ ((packed));
 } ospf_hello_t;
 
 /* ************************************************************
